@@ -50,6 +50,25 @@ DEPLOYING_INTERFACE_PRIORITY = {
     'raid': 1,
 }
 
+VERIFYING_INTERFACE_PRIORITY = {
+    # When two verify steps have the same priority, their order is determined
+    # by which interface is implementing the verify step. The verifying step of
+    # the interface with the highest value here, will be executed first in
+    # that case.
+    'power': 12,
+    'management': 11,
+    'boot': 8,
+    'inspect': 10,
+    'deploy': 9,
+    'bios': 7,
+    'raid': 6,
+    'vendor': 5,
+    'network': 4,
+    'storage': 3,
+    'console': 2,
+    'rescue': 1,
+}
+
 
 def _clean_step_key(step):
     """Sort by priority, then interface priority in event of tie.
@@ -69,6 +88,15 @@ def _deploy_step_key(step):
             DEPLOYING_INTERFACE_PRIORITY[step.get('interface')])
 
 
+def _verify_step_key(step):
+    """Sort by priority, then interface priority in event of tie.
+
+    :param step: verify step dict to get priority for.
+    """
+    return (step.get('priority'),
+            VERIFYING_INTERFACE_PRIORITY[step.get('interface')])
+
+
 def _sorted_steps(steps, sort_step_key):
     """Return a sorted list of steps.
 
@@ -166,9 +194,9 @@ def _get_cleaning_steps(task, enabled=False, sort=True):
                                     sort_step_key=sort_key,
                                     prio_overrides=csp_override)
 
-        LOG.debug("cleaning_steps after applying "
-                  "clean_step_priority_override for node %(node)s: %(step)s",
-                  task.node.uuid, cleaning_steps)
+        LOG.debug('cleaning_steps after applying '
+                  'clean_step_priority_override for node %(node)s: %(steps)s',
+                  {'node': task.node.uuid, 'steps': cleaning_steps})
     else:
         cleaning_steps = _get_steps(task, CLEANING_INTERFACE_PRIORITY,
                                     'get_clean_steps', enabled=enabled,
@@ -194,6 +222,37 @@ def _get_deployment_steps(task, enabled=False, sort=True):
                       enabled=enabled, sort_step_key=sort_key)
 
 
+def _get_verify_steps(task, enabled=False, sort=True):
+    """Get verify steps for task.node.
+
+    :param task: A TaskManager object
+    :param enabled: If True, returns only enabled (priority > 0) steps. If
+        False, returns all verify steps.
+    :param sort: If True, the steps are sorted from highest priority to lowest
+        priority. For steps having the same priority, they are sorted from
+        highest interface priority to lowest.
+    :raises: NodeVerifyFailure if there was a problem getting the
+        verify steps.
+    :returns: A list of verify step dictionaries
+    """
+    sort_key = _verify_step_key if sort else None
+    if CONF.conductor.verify_step_priority_override:
+        vsp_override = {}
+        for element in CONF.conductor.verify_step_priority_override:
+            vsp_override.update(element)
+
+        verify_steps = _get_steps(task, VERIFYING_INTERFACE_PRIORITY,
+                                  'get_verify_steps', enabled=enabled,
+                                  sort_step_key=sort_key,
+                                  prio_overrides=vsp_override)
+
+    else:
+        verify_steps = _get_steps(task, VERIFYING_INTERFACE_PRIORITY,
+                                  'get_verify_steps', enabled=enabled,
+                                  sort_step_key=sort_key)
+    return verify_steps
+
+
 def set_node_cleaning_steps(task, disable_ramdisk=False):
     """Set up the node with clean step information for cleaning.
 
@@ -209,7 +268,6 @@ def set_node_cleaning_steps(task, disable_ramdisk=False):
              clean steps.
     """
     node = task.node
-    driver_internal_info = node.driver_internal_info
 
     # For manual cleaning, the target provision state is MANAGEABLE, whereas
     # for automated cleaning, it is AVAILABLE.
@@ -217,25 +275,24 @@ def set_node_cleaning_steps(task, disable_ramdisk=False):
 
     if not manual_clean:
         # Get the prioritized steps for automated cleaning
-        driver_internal_info['clean_steps'] = _get_cleaning_steps(task,
-                                                                  enabled=True)
+        steps = _get_cleaning_steps(task, enabled=True)
     else:
         # For manual cleaning, the list of cleaning steps was specified by the
         # user and already saved in node.driver_internal_info['clean_steps'].
         # Now that we know what the driver's available clean steps are, we can
         # do further checks to validate the user's clean steps.
-        steps = node.driver_internal_info['clean_steps']
-        driver_internal_info['clean_steps'] = _validate_user_clean_steps(
-            task, steps, disable_ramdisk=disable_ramdisk)
+        steps = _validate_user_clean_steps(
+            task, node.driver_internal_info['clean_steps'],
+            disable_ramdisk=disable_ramdisk)
 
     LOG.debug('List of the steps for %(type)s cleaning of node %(node)s: '
               '%(steps)s', {'type': 'manual' if manual_clean else 'automated',
                             'node': node.uuid,
-                            'steps': driver_internal_info['clean_steps']})
+                            'steps': steps})
 
     node.clean_step = {}
-    driver_internal_info['clean_step_index'] = None
-    node.driver_internal_info = driver_internal_info
+    node.set_driver_internal_info('clean_steps', steps)
+    node.set_driver_internal_info('clean_step_index', None)
     node.save()
 
 
@@ -367,17 +424,16 @@ def set_node_deployment_steps(task, reset_current=True, skip_missing=False):
              deployment steps.
     """
     node = task.node
-    driver_internal_info = node.driver_internal_info
-    driver_internal_info['deploy_steps'] = _get_all_deployment_steps(
-        task, skip_missing=skip_missing)
+    node.set_driver_internal_info('deploy_steps', _get_all_deployment_steps(
+        task, skip_missing=skip_missing))
 
-    LOG.debug('List of the deploy steps for node %(node)s: '
-              '%(steps)s', {'node': node.uuid,
-                            'steps': driver_internal_info['deploy_steps']})
+    LOG.debug('List of the deploy steps for node %(node)s: %(steps)s', {
+        'node': node.uuid,
+        'steps': node.driver_internal_info['deploy_steps']
+    })
     if reset_current:
         node.deploy_step = {}
-        driver_internal_info['deploy_step_index'] = None
-    node.driver_internal_info = driver_internal_info
+        node.set_driver_internal_info('deploy_step_index', None)
     node.save()
 
 
